@@ -1,4 +1,4 @@
-package Pcore::API::Majestic v0.3.0;
+package Pcore::API::Majestic v0.4.0;
 
 use Pcore -dist, -class, -const, -export => { CONST => [qw[$MAJESTIC_INDEX_FRESH $MAJESTIC_INDEX_HISTORIC]] };
 use Pcore::API::Response;
@@ -6,11 +6,22 @@ use Pcore::API::Response;
 const our $MAJESTIC_INDEX_FRESH    => 1;
 const our $MAJESTIC_INDEX_HISTORIC => 2;
 
-has api_key              => ( is => 'ro', isa => Maybe [Str] );
-has openapp_key          => ( is => 'ro', isa => Maybe [Str] );
-has openapp_access_token => ( is => 'ro', isa => Maybe [Str] );
+has api_key              => ( is => 'ro', isa => Maybe [Str] );    # direct access to the API, access is restricted by IP address
+has openapp_access_token => ( is => 'ro', isa => Maybe [Str] );    # OpenApp access, user key, identify user
+has openapp_private_key  => ( is => 'ro', isa => Maybe [Str] );    # OpenApp access, application vendor key, identify application
 has bind_ip              => ( is => 'ro', isa => Maybe [Str] );
 
+sub get_subscription_info ( $self, $cb ) {
+    my $url_params = {
+        cmd        => 'GetSubscriptionInfo',
+        datasource => 'historic',
+    };
+
+    $self->_request( $url_params, $cb );
+
+    return;
+}
+
 sub get_index_item_info ( $self, $domains, $datasource, $failover, $cb ) {
     die q[Maximum items number is 100] if $domains->@* > 100;
 
@@ -21,20 +32,41 @@ sub get_index_item_info ( $self, $domains, $datasource, $failover, $cb ) {
         items                      => scalar $domains->@*,
     };
 
+    for my $i ( 0 .. $domains->$#* ) {
+        $url_params->{ 'item' . $i } = $domains->[$i];
+    }
+
+    $self->_request(
+        $url_params,
+        sub ($res) {
+            if ( $res->is_success ) {
+                my $json = delete $res->{result};
+
+                for my $item ( $json->{DataTables}->{Results}->{Data}->@* ) {
+                    $res->{result}->{ $domains->[ $item->{ItemNum} ] } = $item;
+                }
+            }
+
+            $cb->($res);
+
+            return;
+        }
+    );
+
+    return;
+}
+
+sub _request ( $self, $url_params, $cb ) {
     if ( $self->api_key ) {
         $url_params->{app_api_key} = $self->api_key;
     }
-    elsif ( $self->openapp_key && $self->openapp_access_token ) {
-        $url_params->{privatekey} = $self->openapp_key;
-
+    elsif ( $self->openapp_private_key && $self->openapp_access_token ) {
         $url_params->{accesstoken} = $self->openapp_access_token;
+
+        $url_params->{privatekey} = $self->openapp_private_key;
     }
     else {
-        die q["app_api_key" or "openapp_key" and "openapp_access_token" are missed];
-    }
-
-    for my $i ( 0 .. $domains->$#* ) {
-        $url_params->{ 'item' . $i } = $domains->[$i];
+        die q["api_key" or "openapp_private_key" and "openapp_access_token" are missed];
     }
 
     my $url = 'http://api.majestic.com/api/json?' . P->data->to_uri($url_params);
@@ -61,9 +93,7 @@ sub get_index_item_info ( $self, $domains, $datasource, $failover, $cb ) {
                 else {
                     $api_res = Pcore::API::Response->new( { status => 200 } );
 
-                    for my $item ( $json->{DataTables}->{Results}->{Data}->@* ) {
-                        $api_res->{result}->{ $domains->[ $item->{ItemNum} ] } = $item;
-                    }
+                    $api_res->{result} = $json;
                 }
             }
 
